/* datafile.cpp
 *
 * Pieter Eendebak <pte@ddsw.nl>
 *
 */

#include <stdlib.h>

#include <qobject.h>
#include <qstring.h>
#include <qfile.h>
#include <qdstream.h>
#include <qtstream.h>
#include <qdir.h>

#include <kapp.h>
#include <klocale.h>

#include "defines.h"

#include "datafile.moc"

DataFile::DataFile ( const char *_dir, const char *_file,
			QObject *parent, const char *n )
		: QObject ( parent, n ), number( 0 )
{
	file=new QFile();
	cache=new QFile();
	
	setName( _dir, _file );
}

DataFile::~DataFile()
{
	delete file;
	delete cache;
}

void DataFile::setName( const char *_dir, const char * _file )
{
	if ( _file!=0 && strcmp (_file, "" ) )
	{
		name.sprintf( "%s", _file);

		QString tmp;
		tmp.sprintf( "%s%s", _dir, _file);
		file->setName( tmp.data() );

		tmp.sprintf( "%s%s",
			getenv("HOME"),
			"/.kde/share/apps/kfortune/" );
		QDir d( tmp.data() );
		if ( !d.exists() && !d.mkdir( tmp.data() ) )
		{
			printf("Could not make directory %s.\n", tmp.data() );
			printf("Exiting program...\n");
			exit(1);
		}

		tmp.append( _file );
		tmp.append(".cache");
		cache->setName( tmp.data() );

		if ( !cache->exists() )
		{
			updateCache();
		}

		number = getNumberCache();
	}
}


int DataFile::getNumber()
{
	return number;
}

int DataFile::getNumberCache()
{
	QString tmp;
	int i=0;

	if ( cache->open( IO_ReadOnly ) )
	{
		QTextStream t ( cache );
		t.readLine();
		tmp = t.readLine();
		
		i = tmp.toInt();
		cache->close();
	}

	return i;
}

void DataFile::updateCache()
{
	// here we should remake the cache file, which includes
	// information on the number of fortunes
	// and to place where fortunes are kept...
	
	printf("Updating cache : %s\n", name.data() );
	QString tmp;
	int i;
	int num = 0;

	int d=0;
	int data[1000];
	
	if ( cache->open( IO_WriteOnly ) )
	{
		tmp.sprintf("# Cache file for KFortune %s : %s\n",
				VERSION, name.data() );
		cache->writeBlock( tmp.data(), tmp.length() );


		if (file->open(IO_ReadOnly) )
		{
			QTextStream t ( file );
			QString s;

			while (	!t.eof() )
			{
				s=t.readLine(); 
				if ( !strcmp( (const char *)s , "%") )
				{
					num++;
					if ( num%CACHENUMBER == 0 )
					{
						data[d]=file->at();
						d++;
					}
				}
			}
			file->close();
		}

		tmp.setNum( num );
		tmp.append("\n");
		cache->writeBlock( tmp.data(), tmp.length() );

		cache->writeBlock("# Data\n", 7);

		// streamposition of fortune number 0
		cache->writeBlock("0\n",2);

		for(i=0;i<d;i++)
		{
			// write cache data...
			tmp.setNum( data[i] );
			tmp.append("\n");
			cache->writeBlock( tmp.data(), tmp.length() );
		}
		
		cache->flush();
		cache->close();
	}

}

QString DataFile::getFortune(int nr)
{
	if ( (nr<0) || nr>=number )
	{
		QString t = klocale->translate("Sorry, the program tried to get"
				"a fortune which doesn't exist.\n"
				"Please update your cache-files.");
		return QString(t.data() );
	}

	QString temp;
	int x = (int)(nr/CACHENUMBER);
	int y=0;	
	int tmp=0;

	if (file->open(IO_ReadOnly) )
	{

		// New code for caching... 
		y = getCache( x );
		tmp+= x*CACHENUMBER;
		file->at(y);


		QString s;
		QTextStream t ( file );

		while ( tmp!=nr )
		{
			s=t.readLine();
			if ( !strcmp( s.data() , "%") ) tmp++;
		}

		s = t.readLine();

		do
		{
			temp.append( s.data() );
			temp.append("\n");
			s=t.readLine();
		}
		while ( strcmp( s.data() , "%") && !t.eof() );
		
		file->close();
	}

	return QString( temp.data() );
}

int DataFile::getCache(int x)
{
	QString p;
	
	// get the appropiate streamposition....
 
	if (cache->open(IO_ReadOnly) )
	{
		QTextStream t(cache);
		t.readLine();	// comment line
		// number of fortunes (no check because this is
		// an internal function)
		t.readLine();
		// another comment line
		t.readLine();

		for (int i=0;i<x;i++) t.readLine();
		p = t.readLine();
		
		cache->close();
	}

	return p.toInt();
}

void DataFile::addFortune (QString)
{
	return;
}
